@@ -1,14 +1,14 @@
 /* md.c  -  message digest dispatcher
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -256,6 +256,7 @@ md_copy( MD_HANDLE a )
 		  : m_alloc( sizeof *b );
     memcpy( b, a, sizeof *a );
     b->list = NULL;
+    b->debug = NULL;
     /* and now copy the complete list of algorithms */
     /* I know that the copied list is reversed, but that doesn't matter */
     for( ar=a->list; ar; ar = ar->next ) {
@@ -270,7 +271,7 @@ md_copy( MD_HANDLE a )
 
 
 /****************
- * Reset all contexts and discard any buffered stuuf.  This may be used
+ * Reset all contexts and discard any buffered stuff.  This may be used
  * instead of a md_close(); md_open().
  */
 void
