@@ -1,14 +1,14 @@
 /* random.c  -	random number generator
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -32,6 +32,18 @@
 #include <assert.h>
 #include <errno.h>
 #include <string.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#ifdef	HAVE_GETHRTIME
+  #include <sys/times.h>
+#endif
+#ifdef HAVE_GETTIMEOFDAY
+  #include <sys/times.h>
+#endif
+#ifdef HAVE_GETRUSAGE
+  #include <sys/resource.h>
+#endif
 #include "util.h"
 #include "rmd.h"
 #include "ttyio.h"
@@ -82,10 +94,9 @@ static int faked_rng;
 static void read_pool( byte *buffer, size_t length, int level );
 static void add_randomness( const void *buffer, size_t length, int source );
 static void random_poll(void);
-static void read_random_source( byte *buffer, size_t length, int level );
-#ifndef HAVE_DEV_RANDOM
-static int gather_faked( byte *buffer, size_t *r_length, int level );
-#endif
+static void read_random_source( int requester, size_t length, int level);
+static int gather_faked( void (*add)(const void*, size_t, int), int requester,
+						    size_t length, int level );
 
 
 static void
@@ -127,7 +138,7 @@ quick_random_gen( int onoff )
 {
     int last;
 
-    read_random_source( NULL, 0, 0 ); /* load module */
+    read_random_source(0,0,0); /* init */
     last = quick_test;
     if( onoff != -1 )
 	quick_test = onoff;
@@ -226,17 +237,13 @@ read_pool( byte *buffer, size_t length, int level )
     /* for level 2 make sure that there is enough random in the pool */
     if( level == 2 && pool_balance < length ) {
 	size_t needed;
-	byte *p;
 
 	if( pool_balance < 0 )
 	    pool_balance = 0;
 	needed = length - pool_balance;
 	if( needed > POOLSIZE )
 	    BUG();
-	p = secure_alloc ? m_alloc_secure( needed ) : m_alloc(needed);
-	read_random_source( p, needed, 2 ); /* read /dev/random */
-	add_randomness( p, needed, 3);
-	m_free(p);
+	read_random_source( 3, needed, 2 );
 	pool_balance += needed;
     }
 
@@ -311,17 +318,14 @@ add_randomness( const void *buffer, size_t length, int source )
 static void
 random_poll()
 {
-    char buf[POOLSIZE/5];
-    read_random_source( buf, POOLSIZE/5, 1 );
-    add_randomness( buf, POOLSIZE/5, 2);
-    memset( buf, 0, POOLSIZE/5);
+    read_random_source( 2, POOLSIZE/5, 1 );
 }
 
 
 void
 fast_random_poll()
 {
-    static void (*fnc)( void (*)(const void*, size_t, int)) = NULL;
+    static void (*fnc)( void (*)(const void*, size_t, int), int) = NULL;
     static int initialized = 0;
 
     if( !initialized ) {
@@ -329,19 +333,48 @@ fast_random_poll()
 	    initialize();
 	initialized = 1;
 	fnc = dynload_getfnc_fast_random_poll();
-	if( !fnc )
-	    log_info("Ooops: No fast random poll function\n");
     }
-    if( fnc )
-	(*fnc)( add_randomness );
+    if( fnc ) {
+	(*fnc)( add_randomness, 1 );
+	return;
+    }
+
+    /* fall back to the generic function */
+  #if HAVE_GETHRTIME
+    {	hrtime_t tv;
+	tv = gethrtime();
+	add_randomness( &tv, sizeof(tv), 1 );
+    }
+  #elif HAVE_GETTIMEOFDAY
+    {	struct timeval tv;
+	if( gettimeofday( &tv, NULL ) )
+	    BUG();
+	add_randomness( &tv.tv_sec, sizeof(tv.tv_sec), 1 );
+	add_randomness( &tv.tv_usec, sizeof(tv.tv_usec), 1 );
+    }
+  #else /* use times */
+    {	struct tms buf;
+	times( &buf );
+	add_randomness( &buf, sizeof buf, 1 );
+    }
+  #endif
+  #ifdef HAVE_GETRUSAGE
+    {	struct rusage buf;
+	if( getrusage( RUSAGE_SELF, &buf ) )
+	    BUG();
+	add_randomness( &buf, sizeof buf, 1 );
+	memset( &buf, 0, sizeof buf );
+    }
+  #endif
 }
 
 
 
 static void
-read_random_source( byte *buffer, size_t length, int level )
+read_random_source( int requester, size_t length, int level )
 {
-    static int (*fnc)(byte*, size_t*, int) = NULL;
+    static int (*fnc)(void (*)(const void*, size_t, int), int,
+						    size_t, int) = NULL;
     int nbytes;
     int goodness;
 
@@ -351,34 +384,28 @@ read_random_source( byte *buffer, size_t length, int level )
 	fnc = dynload_getfnc_gather_random();
 	if( !fnc ) {
 	    faked_rng = 1;
-	  #ifndef HAVE_DEV_RANDOM
 	    fnc = gather_faked;
-	  #else
-	    BUG();
-	  #endif
 	}
+	if( !requester && !length && !level )
+	    return; /* init only */
     }
-    while( length ) {
-	nbytes = length;
-	goodness = (*fnc)( buffer, &nbytes, level );
-	buffer +=nbytes;
-	length -= nbytes;
-	/* FIXME: how can we handle the goodness */
-    }
+    if( (*fnc)( add_randomness, requester, length, level ) < 0 )
+	log_fatal("No way to gather entropy for the RNG\n");
 }
 
 
-#ifndef HAVE_DEV_RANDOM
 static int
-gather_faked( byte *buffer, size_t *r_length, int level )
+gather_faked( void (*add)(const void*, size_t, int), int requester,
+	      size_t length, int level )
 {
     static int initialized=0;
-    size_t length = *r_length;
+    size_t n;
+    char *buffer, *p;
 
     if( !initialized ) {
 	log_info(_("WARNING: using insecure random number generator!!\n"));
 	tty_printf(_("The random number generator is only a kludge to let\n"
-		   "it compile - it is in no way a strong RNG!\n\n"
+		   "it run - it is in no way a strong RNG!\n\n"
 		   "DON'T USE ANY DATA GENERATED BY THIS PROGRAM!!\n\n"));
 	initialized=1;
       #ifdef HAVE_RAND
@@ -388,15 +415,17 @@ gather_faked( byte *buffer, size_t *r_length, int level )
       #endif
     }
 
+    p = buffer = m_alloc( length );
+    n = length;
   #ifdef HAVE_RAND
-    while( length-- )
-	*buffer++ = ((unsigned)(1 + (int) (256.0*rand()/(RAND_MAX+1.0)))-1);
+    while( n-- )
+	*p++ = ((unsigned)(1 + (int) (256.0*rand()/(RAND_MAX+1.0)))-1);
   #else
-    while( length-- )
-	*buffer++ = ((unsigned)(1 + (int) (256.0*random()/(RAND_MAX+1.0)))-1);
+    while( n-- )
+	*p++ = ((unsigned)(1 + (int) (256.0*random()/(RAND_MAX+1.0)))-1);
   #endif
-    return 100; /* We really fake it ;-) */
+    add_randomness( buffer, length, requester );
+    m_free(buffer);
+    return 0; /* okay */
 }
 
-#endif /* ! HAVE_DEV_RANDOM */
-
