@@ -454,6 +454,7 @@ find_header( fhdr_state_t state, byte *buf, size_t *r_buflen,
 	    /* found the begin line */
 	    hdr_line = i;
 	    state = fhdrWAITHeader;
+	    *not_dashed = 0;
 	    if( hdr_line == BEGIN_SIGNED_MSG_IDX )
 		clearsig = 1;
 	    if( opt.verbose > 1 )
@@ -1079,7 +1080,7 @@ armor_filter( void *opaque, int control,
 	    }
 	    else
 		iobuf_writestr(a,
-		    "Comment: For info finger gcrypt@ftp.guug.de\n");
+		    "Comment: For info see www.gnupg.org");
 	    if( afx->hdrlines )
 		iobuf_writestr(a, afx->hdrlines);
 	    iobuf_put(a, '\n');
@@ -1179,7 +1180,7 @@ armor_filter( void *opaque, int control,
 	    iobuf_writestr(a, "-----\n");
 	}
 	else if( !afx->any_data && !afx->inp_bypass )
-	    log_error(_("no valid RFC1991 or OpenPGP data found.\n"));
+	    log_error(_("no valid OpenPGP data found.\n"));
     }
     else if( control == IOBUFCTRL_DESC )
 	*(char**)buf = "armor_filter";
