@@ -1,14 +1,14 @@
 /* g10.c - The GnuPG utility (main for gpg)
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -84,6 +84,7 @@ enum cmd_and_opt_values { aNull = 0,
     aListSigs,
     aListSecretKeys,
     aExport,
+    aExportAll,
     aExportSecret,
     aCheckKeys,
     aGenRevoke,
@@ -149,6 +150,7 @@ enum cmd_and_opt_values { aNull = 0,
     oS2KCipher,
     oCharset,
     oNotDashEscaped,
+    oEscapeFrom,
     oLockOnce,
 aTest };
 
@@ -180,7 +182,8 @@ static ARGPARSE_OPTS opts[] = {
     { aEditKey, "edit-key"  ,256, N_("sign or edit a key")},
     { aGenRevoke, "gen-revoke",256, N_("generate a revocation certificate")},
   #endif
-    { aExport, "export"          , 256, N_("export keys") },
+    { aExport, "export"           , 256, N_("export keys") },
+    { aExportAll, "export-all"    , 256, "@" },
     { aExportSecret, "export-secret-keys" , 256, "@" },
     { aImport, "import",      256     , N_("import/merge keys")},
     { aFastImport, "fast-import",  256 , "@"},
@@ -299,6 +302,7 @@ static ARGPARSE_OPTS opts[] = {
     { oSetFilename, "set-filename", 2, "@" },
     { oComment, "comment", 2, "@" },
     { oNotDashEscaped, "not-dash-escaped", 0, "@" },
+    { oEscapeFrom, "escape-from-lines", 0, "@" },
     { oLockOnce, "lock-once", 0, "@" },
 {0} };
 
@@ -634,6 +638,7 @@ main( int argc, char **argv )
 	  case aImport: set_cmd( &cmd, aImport); break;
 	  case aFastImport: set_cmd( &cmd, aFastImport); break;
 	  case aExport: set_cmd( &cmd, aExport); break;
+	  case aExportAll: set_cmd( &cmd, aExportAll); break;
 	  case aListKeys: set_cmd( &cmd, aListKeys); break;
 	  case aListSigs: set_cmd( &cmd, aListSigs); break;
 	  case aExportSecret: set_cmd( &cmd, aExportSecret); break;
@@ -724,7 +729,11 @@ main( int argc, char **argv )
 	    register_cipher_extension(orig_argc? *orig_argv:NULL,
 				      pargs.r.ret_str);
 	    break;
-	  case oRFC1991: opt.rfc1991 = 1; opt.no_comment = 1; break;
+	  case oRFC1991:
+	    opt.rfc1991 = 1;
+	    opt.no_comment = 1;
+	    opt.escape_from = 1;
+	    break;
 	  case oEmuChecksumBug: opt.emulate_bugs |= EMUBUG_GPGCHKSUM; break;
 	  case oCompressSigs: opt.compress_sigs = 1; break;
 	  case oRunAsShmCP:
@@ -774,6 +783,7 @@ main( int argc, char **argv )
 						    pargs.r.ret_str);
 	    break;
 	  case oNotDashEscaped: opt.not_dash_escaped = 1; break;
+	  case oEscapeFrom: opt.escape_from = 1; break;
 	  case oLockOnce: opt.lock_once = 1; break;
 
 	  default : pargs.err = configfp? 1:2; break;
@@ -1084,10 +1094,11 @@ main( int argc, char **argv )
 	break;
 
       case aExport:
+      case aExportAll:
 	sl = NULL;
 	for( ; argc; argc--, argv++ )
 	    add_to_strlist( &sl, *argv );
-	export_pubkeys( sl );
+	export_pubkeys( sl, (cmd == aExport) );
 	free_strlist(sl);
 	break;
 
@@ -1315,17 +1326,6 @@ g10_exit( int rc )
 }
 
 
-void
-do_not_use_RSA()
-{
-    static int did_rsa_note = 0;
-
-    if( !did_rsa_note ) {
-	did_rsa_note = 1;
-	log_info(_("RSA keys are deprecated; please consider "
-		   "creating a new key and use this key in the future\n"));
-    }
-}
 
 
 #ifdef IS_G10MAINT
