@@ -1,14 +1,14 @@
 /* main.h
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -45,7 +45,9 @@ extern int g10_errors_seen;
 #else
   void g10_exit(int rc);
 #endif
-void do_not_use_RSA(void);
+void print_pubkey_algo_note( int algo );
+void print_cipher_algo_note( int algo );
+void print_digest_algo_note( int algo );
 
 /*-- misc.c --*/
 void trap_unaligned(void);
@@ -108,7 +110,7 @@ KBNODE make_mpi_comment_node( const char *s, MPI a );
 /*-- import.c --*/
 int import_keys( const char *filename, int fast );
 /*-- export.c --*/
-int export_pubkeys( STRLIST users );
+int export_pubkeys( STRLIST users, int onlyrfc );
 int export_seckeys( STRLIST users );
 
 /* dearmor.c --*/
