@@ -1,14 +1,14 @@
 /* misc.c -  miscellaneous functions
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#if defined(__linux__) && defined(__alpha__)
+#if defined(__linux__) && defined(__alpha__) && __GLIBC__ < 2
   #include <asm/sysinfo.h>
   #include <asm/unistd.h>
 #endif
@@ -50,7 +50,7 @@ pull_in_libs(void)
 }
 
 
-#if defined(__linux__) && defined(__alpha__)
+#if defined(__linux__) && defined(__alpha__) && __GLIBC__ < 2
 #warning using trap_unaligned
 static int
 setsysinfo(unsigned long op, void *buffer, unsigned long size,
@@ -187,3 +187,61 @@ buffer_to_u32( const byte *buffer )
     return a;
 }
 
+
+static void
+no_exp_algo(void)
+{
+    static int did_note = 0;
+
+    if( !did_note ) {
+	did_note = 1;
+	log_info(_("Experimental algorithms should not be used!\n"));
+    }
+}
+
+void
+print_pubkey_algo_note( int algo )
+{
+    if( algo >= 100 && algo <= 110 )
+	no_exp_algo();
+    else if( is_RSA( algo ) ) {
+	static int did_note = 0;
+
+	if( !did_note ) {
+	    did_note = 1;
+	    log_info(_("RSA keys are deprecated; please consider "
+		       "creating a new key and use this key in the future\n"));
+	}
+    }
+}
+
+void
+print_cipher_algo_note( int algo )
+{
+    if( algo >= 100 && algo <= 110 )
+	no_exp_algo();
+    else if(	algo == CIPHER_ALGO_3DES
+	     || algo == CIPHER_ALGO_CAST5
+	     || algo == CIPHER_ALGO_BLOWFISH
+	   )
+	;
+    else {
+	static int did_note = 0;
+
+	if( !did_note ) {
+	    did_note = 1;
+	    log_info(_("This cipher algorithm is depreciated; "
+		       "please use a more standard one!\n"));
+	}
+    }
+}
+
+void
+print_digest_algo_note( int algo )
+{
+    if( algo >= 100 && algo <= 110 )
+	no_exp_algo();
+}
+
+
+
