@@ -1,14 +1,14 @@
 /* pkclist.c
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -77,6 +77,22 @@ show_paths( ulong lid, int only_first )
 		  level*2, "",
 		  nbits_from_pk( pk ), pubkey_letter( pk->pubkey_algo ),
 		  (ulong)keyid[1], lid, datestr_from_pk( pk ) );
+     #if 0
+	c = trust_letter(otrust);
+	if( c )
+	    putchar( c );
+	else
+	    printf( "%02x", otrust );
+	putchar('/');
+	c = trust_letter(validity);
+	if( c )
+	    putchar( c );
+	else
+	    printf( "%02x", validity );
+	putchar(' ');
+      #endif
+
+
 	p = get_user_id( keyid, &n );
 	tty_print_string( p, n ),
 	m_free(p);
@@ -102,6 +118,7 @@ edit_ownertrust( ulong lid, int mode )
     u32 keyid[2];
     PKT_public_key *pk ;
     int changed=0;
+    int quit=0;
 
     rc = keyid_from_lid( lid, keyid );
     if( rc ) {
@@ -138,13 +155,15 @@ edit_ownertrust( ulong lid, int mode )
 " s = please show me more information\n") );
     if( mode )
 	tty_printf(_(" m = back to the main menu\n"));
+    else
+	tty_printf(_(" q = quit\n"));
     tty_printf("\n");
 
     for(;;) {
 	/* a string with valid answers */
-	char *ans = _("sSmM");
+	char *ans = _("sSmMqQ");
 
-	if( strlen(ans) != 4 )
+	if( strlen(ans) != 6 )
 	    BUG();
 	p = cpr_get("edit_ownertrust.value",_("Your decision? "));
 	trim_spaces(p);
@@ -172,20 +191,26 @@ edit_ownertrust( ulong lid, int mode )
 	else if( mode && (*p == ans[2] || *p == ans[3] || *p == CONTROL_D ) ) {
 	    break ; /* back to the menu */
 	}
+	else if( !mode && (*p == ans[4] || *p == ans[5] ) ) {
+	    quit = 1;
+	    break ; /* back to the menu */
+	}
 	m_free(p); p = NULL;
     }
     m_free(p);
     m_free(pk);
-    return changed;
+    return quit? -1 : changed;
 }
 
 
 /****************
  * Try to add some more owner trusts (interactive)
+ * This function presents all the signator in a certificate
+ * chain who have no trust value assigned.
  * Returns: -1 if no ownertrust were added.
  */
 static int
-add_ownertrust( PKT_public_key *pk )
+add_ownertrust( PKT_public_key *pk, int *quit )
 {
     int rc;
     void *context = NULL;
@@ -193,6 +218,7 @@ add_ownertrust( PKT_public_key *pk )
     unsigned otrust, validity;
     int any=0, changed=0, any_undefined=0;
 
+    *quit = 0;
     tty_printf(
 _("Could not find a valid trust path to the key.  Let's see whether we\n"
   "can assign some missing owner trust values.\n\n"));
@@ -205,12 +231,28 @@ _("Could not find a valid trust path to the key.  Let's see whether we\n"
 
     lid = pk->local_id;
     while( enum_cert_paths( &context, &lid, &otrust, &validity ) != -1 ) {
+	if( lid == pk->local_id )
+	    continue;
 	any=1;
-	if( otrust == TRUST_UNDEFINED || otrust == TRUST_EXPIRED ||
-	    otrust == TRUST_UNKNOWN ) {
+	if( changed ) {
+	    /* because enum_cert_paths() makes a snapshop of the
+	     * trust paths, the otrust and validity are not anymore
+	     * valid after changing an entry - we have to reread
+	     * those values from then on
+	     */
+	    otrust = get_ownertrust( lid );
+	}
+	if( otrust == TRUST_UNDEFINED ) {
 	    any_undefined=1;
-	    if( edit_ownertrust( lid, 0 ) )
-		changed=1;
+	    enum_cert_paths_print( &context, NULL, changed, lid );
+	    tty_printf("\n");
+	    rc = edit_ownertrust( lid, 0 );
+	    if( rc == -1 ) {
+		*quit = 1;
+		break;
+	    }
+	    else if( rc > 0 )
+	       changed = 1;
 	}
     }
     enum_cert_paths( &context, NULL, NULL, NULL ); /* release context */
@@ -272,8 +314,10 @@ do_we_trust( PKT_public_key *pk, int trustlevel )
 	    log_info(_("%08lX: no info to calculate a trust probability\n"),
 					(ulong)keyid_from_pk( pk, NULL) );
 	else {
-	    rc = add_ownertrust( pk );
-	    if( !rc ) {
+	    int quit;
+
+	    rc = add_ownertrust( pk, &quit );
+	    if( !rc && !quit ) {
 		rc = check_trust( pk, &trustlevel );
 		if( rc )
 		    log_fatal("trust check after add_ownertrust failed: %s\n",
@@ -302,7 +346,7 @@ do_we_trust( PKT_public_key *pk, int trustlevel )
 
       case TRUST_ULTIMATE:
 	if( opt.verbose )
-	    log_info(_("This key belongs to us (we have the secret key)\n"));
+	    log_info(_("This key belongs to us\n"));
 	return 1; /* yes */
 
       default: BUG();
@@ -407,8 +451,9 @@ check_signatures_trust( PKT_signature *sig )
 				    "signature belongs to the owner.\n" ));
 	}
 	else {
-	    rc = add_ownertrust( pk );
-	    if( rc ) {
+	    int quit;
+	    rc = add_ownertrust( pk, &quit );
+	    if( rc || quit ) {
 		dont_try = 1;
 		rc = 0;
 	    }
@@ -595,12 +640,14 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 	if( !pkr->pk->local_id )
 	    BUG(); /* if this occurs, we can use get_ownertrust to set it */
 	if( preftype == PREFTYPE_SYM )
-	    bits[0] = (1<<2); /* 3DES is implicitly there */
+	    mask[0] |= (1<<2); /* 3DES is implicitly there */
 	m_free(pref);
 	pref = get_pref_data( pkr->pk->local_id, pkr->pk->namehash, &npref);
 	any = 0;
 	if( pref ) {
-	    /*log_hexdump("raw: ", pref, npref );*/
+	   #if 0
+	    log_hexdump("raw: ", pref, npref );
+	   #endif
 	    for(i=0; i+1 < npref; i+=2 ) {
 		if( pref[i] == preftype ) {
 		    mask[pref[i+1]/32] |= 1 << (pref[i+1]%32);
@@ -613,14 +660,18 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 	    compr_hack = 1;
 	}
 
-	/*log_debug("mask=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
+      #if 0
+	log_debug("mask=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
 	       (ulong)mask[7], (ulong)mask[6], (ulong)mask[5], (ulong)mask[4],
-	     (ulong)mask[3], (ulong)mask[2], (ulong)mask[1], (ulong)mask[0]);*/
+	     (ulong)mask[3], (ulong)mask[2], (ulong)mask[1], (ulong)mask[0]);
+      #endif
 	for(i=0; i < 8; i++ )
 	    bits[i] &= mask[i];
-	/*log_debug("bits=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
+      #if 0
+	log_debug("bits=%08lX%08lX%08lX%08lX%08lX%08lX%08lX%08lX\n",
 	       (ulong)bits[7], (ulong)bits[6], (ulong)bits[5], (ulong)bits[4],
-	     (ulong)bits[3], (ulong)bits[2], (ulong)bits[1], (ulong)bits[0]);*/
+	     (ulong)bits[3], (ulong)bits[2], (ulong)bits[1], (ulong)bits[0]);
+      #endif
     }
     /* usable algorithms are now in bits
      * We now use the last key from pk_list to select
@@ -635,6 +686,7 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
 	    if( pref[j] == preftype ) {
 		any = 1;
 		if( (bits[pref[j+1]/32] & (1<<(pref[j+1]%32))) ) {
+		    /* fixme: check whether this algoritm is available */
 		    i = pref[j+1];
 		    break;
 		}
@@ -644,11 +696,14 @@ select_algo_from_prefs( PK_LIST pk_list, int preftype )
     if( !pref || !any ) {
 	for(j=0; j < 256; j++ )
 	    if( (bits[j/32] & (1<<(j%32))) ) {
+		/* fixme: check whether this algoritm is available */
 		i = j;
 		break;
 	    }
     }
-    /*log_debug("prefs of type %d: selected %d\n", preftype, i );*/
+  #if 0
+    log_debug("prefs of type %d: selected %d\n", preftype, i );
+  #endif
     if( compr_hack && !i ) {
 	/* selected no compression, but we should check whether
 	 * algorithm 1 is also available (the ordering is not relevant
