@@ -1,14 +1,14 @@
 /* seckey-cert.c -  secret key certifucate packet handling
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -209,6 +209,7 @@ protect_secret_key( PKT_secret_key *sk, DEK *dek )
 	else if( cipher_get_blocksize( sk->protect.algo ) != 8 )
 	    rc = G10ERR_CIPHER_ALGO; /* unsupport protection algorithm */
 	else {
+	    print_cipher_algo_note( sk->protect.algo );
 	    cipher_hd = cipher_open( sk->protect.algo,
 				     CIPHER_MODE_AUTO_CFB, 1 );
 	    if( cipher_setkey( cipher_hd, dek->key, dek->keylen ) )
