@@ -1,14 +1,14 @@
 /* sign.c - sign data
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -47,12 +47,12 @@ do_sign( PKT_secret_key *sk, PKT_signature *sig,
     byte *dp;
     int rc;
 
-    if( is_RSA(sk->pubkey_algo) )
-	do_not_use_RSA();
+    print_pubkey_algo_note(sk->pubkey_algo);
 
     if( !digest_algo )
 	digest_algo = md_get_algo(md);
 
+    print_digest_algo_note( digest_algo );
     dp = md_read( md, digest_algo );
     sig->digest_algo = digest_algo;
     sig->digest_start[0] = dp[0];
@@ -475,12 +475,35 @@ write_dash_escaped( IOBUF inp, IOBUF out, MD_HANDLE md )
     }
 
     while( (c = iobuf_get(inp)) != -1 ) {
-	/* Note: We don't escape "From " because the MUA should cope with it */
 	if( lastlf ) {
 	    if( c == '-' ) {
 		iobuf_put( out, c );
 		iobuf_put( out, ' ' );
 	    }
+	    else if( c == 'F' && opt.escape_from ) {
+		int i;
+
+		if( state >= 1 )
+		    md_putc(md, '\r');
+		if( state >= 2 )
+		    md_putc(md, '\n');
+		state = 0;
+
+		for( i=1; i < 5 && (c = iobuf_get(inp)) != -1; i++ ) {
+		    if( "From "[i] != c )
+			break;
+		}
+		if( i < 5 ) {
+		    iobuf_write( out, "From", i );
+		    md_write( md, "From", i );
+		    if( c == -1 )
+			break;
+		}
+		else {
+		    iobuf_writestr( out, "- From" );
+		    md_write( md, "From", 4 );
+		}
+	    }
 	}
 
       again:
@@ -512,6 +535,12 @@ write_dash_escaped( IOBUF inp, IOBUF out, MD_HANDLE md )
     }
     if( state == 1 )
 	md_putc(md, '\r');
+    else if( state == 2 ) {  /* file ended with a new line */
+	md_putc(md, '\r');
+	md_putc(md, '\n');
+	iobuf_put( out, '\n');
+    }
+
     if( !lastlf )
 	iobuf_put( out, '\n' );
 
