@@ -1,14 +1,14 @@
 /* status.c
  *	Copyright (C) 1998 Free Software Foundation, Inc.
  *
- * This file is part of GNUPG.
+ * This file is part of GnuPG.
  *
- * GNUPG is free software; you can redistribute it and/or modify
+ * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * GNUPG is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -152,7 +152,8 @@ init_shm_coprocessing ( ulong requested_shm_size, int lock_mem )
 	else
 	    shm_is_locked = 1;
       #elif defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
-	if ( mlock (shm_area, shm_size) )
+	/* (need the cast for Solaris with Sun's workshop compilers) */
+	if ( mlock ( (char*)shm_area, shm_size) )
 	    log_info("locking shared memory %d failed: %s\n",
 				shm_id, strerror(errno));
 	else
@@ -307,7 +308,8 @@ cpr_kill_prompt(void)
     if( opt.shm_coprocess )
 	return;
   #endif
-    return tty_kill_prompt();
+    tty_kill_prompt();
+    return;
 }
 
 int
